// All code is copyright 2004 by George Harth <george@modeblue.com>
// except portions of EventSink.h and EventSink.cpp taken from
// an eSignal sample appliction

// EventSink.cpp : implementation file
//

#include "stdafx.h"
#include "EventSink.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

CESignalConv g_ESignalConv;		// Globally shared conversation object

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

void dump_com_error(_com_error &e)
{
   CString msg;
   msg = "Error initializing!\n\n";

   CString tmp;
   tmp.Format("%s", (PCSTR)e.ErrorMessage());

   msg += tmp + "\n";

   AfxMessageBox(msg, MB_OK, 0);

}

CESignalConv::CESignalConv()
{
	m_piHooks = NULL;
	m_pEvents = NULL;

	::CoInitialize(NULL);
}

CESignalConv::~CESignalConv()
{
	Disconnect();
	::CoUninitialize();
}

bool CESignalConv::Connect(LPCTSTR username)
{
	bool bResult = false;
	m_piHooks = NULL;
	m_pEvents = NULL;

    try {
		m_piHooks = new IHooksPtr("IESignal.Hooks.1");
		m_pEvents = new EventSink(this);
		LPUNKNOWN pUnk = m_pEvents->GetIDispatch(FALSE);

		/*establish the connection*/
		if(AfxConnectionAdvise(*m_piHooks, __uuidof(_IHooksEvents), pUnk, FALSE, &m_dwCookie)) {
			(*m_piHooks)->SetApplication(S2B(username));
			bResult = true;
		}
    } 
    catch (_com_error &e) {
       dump_com_error(e);
    }

	return bResult;
}

void CESignalConv::Disconnect()
{
	long lHandle;
	CTimeSalesConsumer* pConsumer;
	POSITION pos = m_TickMap.GetStartPosition();
	while (pos) {
		m_TickMap.GetNextAssoc(pos, (void*&)lHandle, (void*&)pConsumer);
		if (pConsumer) {
			(*m_piHooks)->ReleaseTimeSales(lHandle);
		}
	}

   if(m_pEvents) {
		LPUNKNOWN pUnk = m_pEvents->GetIDispatch(FALSE);
		AfxConnectionUnadvise(*m_piHooks, __uuidof(_IHooksEvents), pUnk, FALSE, m_dwCookie);
	}

	if(m_piHooks) {
		delete m_piHooks;
		m_piHooks = NULL;
	}

	if(m_pEvents) {
		delete m_pEvents;
		m_pEvents = NULL;
	}
}

bool CESignalConv::IsEntitled()
{
	bool bResult = false;
	if (m_piHooks) {
		bResult = (*m_piHooks)->GetIsEntitled() != 0;
	}
	return bResult;
}

bool CESignalConv::IsHistoryReady(long lHandle)
{
	bool bResult = false;
	if (m_piHooks) {
		bResult = (*m_piHooks)->GetIsHistoryReady(lHandle) != 0;
	}
	return bResult;
}

bool CESignalConv::RequestTimeSales(LPCTSTR symbol, long numDays, bool bQuotes, bool bTrades, 
									CTimeSalesConsumer* pConsumer)
{
	TimeSalesFilter filter;
	filter.bFilterPrice = 0;
	filter.bFilterQuoteExchanges = 0;
	filter.bFilterTradeExchanges = 0;
	filter.bFilterVolume = 0;
	filter.bQuotes = bQuotes ? 1 : 0;
	filter.bTrades = bTrades ? 1 : 0;
	filter.lNumDays = numDays;
	filter.sSymbol = S2B(symbol);

	filter.dMaxPrice = 0.0;
	filter.dMinPrice = 0.0;
	filter.lVolume = 0;
	filter.sQuoteExchangesList = NULL;
	filter.sTradeExchangesList = NULL;

	long lHandle;
	HRESULT hr = (*m_piHooks)->get_RequestTimeSales(&filter, &lHandle);
	if (SUCCEEDED(hr)) {
		m_TickMap.SetAt((void*)lHandle, (void*)pConsumer);
		pConsumer->SetHandle(lHandle);
	}
	else {
		pConsumer->SetHandle(0);
		AfxMessageBox("RequestTimeSales failed");
	}
	return true;
}

void CESignalConv::ES_OnTimeSalesChanged(long lHandle)
{
	CTimeSalesConsumer* pConsumer;
	m_TickMap.Lookup((void*)lHandle, (void*&)pConsumer);
	if (pConsumer) {
		pConsumer->OnTimeSalesChanged(lHandle);
	}
}

bool CESignalConv::GetTimeSalesBar(long lHandle, long index, TimeSalesData& ts)
{
	long numbars = (*m_piHooks)->GetGetNumTimeSalesBars(lHandle);
	if (numbars == 0) {
		return false;
	}

	int rtbarCount = (*m_piHooks)->GetGetNumTimeSalesRtBars(lHandle);
	int historicBars = numbars - rtbarCount;

	if ((index <= -numbars) || (index > 0)) {
		return false;
	}

	ts = (*m_piHooks)->GetGetTimeSalesBar(lHandle, index);
	return true;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// EventSink

IMPLEMENT_DYNCREATE(EventSink, CCmdTarget)

EventSink::EventSink()
{
   m_pListener = NULL;
	EnableAutomation();
}

EventSink::EventSink(EventSinkListener *pListener) 
{
	m_pListener = pListener;
	EnableAutomation();
}

EventSink::~EventSink()
{
}


void EventSink::OnFinalRelease()
{
	// When the last reference for an automation object is released
	// OnFinalRelease is called.  The base class will automatically
	// deletes the object.  Add additional cleanup required for your
	// object before calling the base class.

	CCmdTarget::OnFinalRelease();
}


BEGIN_MESSAGE_MAP(EventSink, CCmdTarget)
	//{{AFX_MSG_MAP(EventSink)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BEGIN_DISPATCH_MAP(EventSink, CCmdTarget)
	//{{AFX_DISPATCH_MAP(EventSink)
      DISP_FUNCTION_ID(EventSink, "OnTimeSalesChanged",		12, OnTimeSalesChanged,		VT_ERROR, VTS_I4)
	//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()

// Note: we add support for IID_IEventSink to support typesafe binding
//  from VBA.  This IID must match the GUID that is attached to the 
//  dispinterface in the .ODL file.

// {A7E17FDF-37B6-48F5-862C-3A5FA1CF7B26}
static const IID IID_IEventSink =
{ 0xa7e17fdf, 0x37b6, 0x48f5, { 0x86, 0x2c, 0x3a, 0x5f, 0xa1, 0xcf, 0x7b, 0x26 } };

BEGIN_INTERFACE_MAP(EventSink, CCmdTarget)
	INTERFACE_PART(EventSink, __uuidof(_IHooksEvents), Dispatch)
END_INTERFACE_MAP()

/////////////////////////////////////////////////////////////////////////////
// EventSink message handlers

SCODE EventSink::OnTimeSalesChanged(long lHandle)
{
   if(m_pListener) {
      m_pListener->ES_OnTimeSalesChanged(lHandle);
   }

   return S_OK;
}

