// All code is copyright 2004 by George Harth <george@modeblue.com>
// except portions of EventSink.h and EventSink.cpp taken from
// an eSignal sample appliction


// TimeAndSalesPane.cpp : implementation file
//

#include "stdafx.h"
#include "ESigTimeAndSalesCpp.h"
#include "TimeAndSalesPane.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTimeAndSalesPane

CString CTimeAndSalesPane::m_PaneWndClass;
CFont CTimeAndSalesPane::m_Font;
CPen CTimeAndSalesPane::m_OutlinePen;

CTimeAndSalesPane::CTimeAndSalesPane()
{
	m_TimeSalesHandle = 0;
	if (m_PaneWndClass.IsEmpty()) {
		m_PaneWndClass = AfxRegisterWndClass(CS_VREDRAW | CS_HREDRAW | CS_DBLCLKS, ::LoadCursor(NULL, IDC_ARROW));
		m_Font.CreatePointFont(100, "Courier New", NULL);
		m_OutlinePen.CreatePen(PS_SOLID, 1, RGB(30, 80, 30));
	}
}

CTimeAndSalesPane::~CTimeAndSalesPane()
{
}


BEGIN_MESSAGE_MAP(CTimeAndSalesPane, CWnd)
	//{{AFX_MSG_MAP(CTimeAndSalesPane)
	ON_WM_PAINT()
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CTimeAndSalesPane message handlers

void CTimeAndSalesPane::SetHandle(long lHandle)
{
	m_TimeSalesHandle = lHandle;
}

void CTimeAndSalesPane::OnTimeSalesChanged(long lHandle)
{
	InvalidateRect(NULL);
}

BOOL CTimeAndSalesPane::QuickCreate(const RECT& rc, CWnd* pParent)
{
	return Create(m_PaneWndClass, NULL, WS_CHILD | WS_VISIBLE, rc, pParent, 0);
}

void CTimeAndSalesPane::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	RenderNow(&dc);
}

void CTimeAndSalesPane::OnSize(UINT nType, int cx, int cy) 
{
	// Force the recreation of the double buffer so it matches
	// the new dimensions.  Recreation occurs in RenderNow()
	m_dc.DeleteDC();
	m_bmp.DeleteObject();
	CWnd::OnSize(nType, cx, cy);
}

void CTimeAndSalesPane::RenderNow(CDC* pDC)
{
	bool bDestroyDC = false;
	if (pDC == NULL) {
		bDestroyDC = true;
		pDC = GetDC();
	}

	CRect clientrc;
	GetClientRect(&clientrc);

	if (m_dc.GetSafeHdc() == NULL) {
		m_dc.CreateCompatibleDC(pDC);
		m_bmp.CreateCompatibleBitmap(pDC, clientrc.right - clientrc.left, clientrc.bottom - clientrc.top);
	}

	// Select back buffer into the device context for rendering
	m_dc.SelectObject(m_bmp);
	m_dc.FillSolidRect(0, 0, clientrc.right, clientrc.bottom, RGB(0, 0, 0));	// Start with solid black background
	m_dc.SelectObject(&m_Font);

	CRect rcDraw = clientrc;
	rcDraw.DeflateRect(5, 5);
	DrawTS(rcDraw);
	DrawOutline(rcDraw);

	// Blit back buffer into view
	pDC->BitBlt(0, 0, clientrc.right, clientrc.bottom, &m_dc, 0, 0, SRCCOPY);
	if (bDestroyDC) {
		ReleaseDC(pDC);
	}
}

void CTimeAndSalesPane::DrawOutline(const CRect& rc)
{
	CPen* oldpen = m_dc.SelectObject(&m_OutlinePen);

	m_dc.MoveTo(rc.left, rc.top);
	m_dc.LineTo(rc.right - 1, rc.top);
	m_dc.LineTo(rc.right - 1, rc.bottom - 1);
	m_dc.LineTo(rc.left, rc.bottom - 1);
	m_dc.LineTo(rc.left, rc.top);

	m_dc.SelectObject(oldpen);
}

void CTimeAndSalesPane::DrawTS(const CRect& rc)
{
	CRgn rgn;
	rgn.CreateRectRgnIndirect(&rc);
	m_dc.SelectClipRgn(&rgn);

	m_dc.SetBkMode(TRANSPARENT);
	if (m_TimeSalesHandle != 0) {
		int x = rc.left + 3;
		int y = rc.top + 1;
		char buf[256];
		int index = 0;
		TimeSalesData ts;
		while (g_ESignalConv.GetTimeSalesBar(m_TimeSalesHandle, index--, ts) && (y < rc.bottom)) {
			double price = ts.dPrice;
			long size = ts.lSize;
			COleDateTime datetime = ts.dtTime;

			if (ts.DataType == tsdTRADE) {
				m_dc.SetTextColor(RGB(255, 255, 255));
				sprintf(buf, "%d-%02d-%02d %02d:%02d:%02d  %.2f  %5d  TRADE", 
					datetime.GetYear(), datetime.GetMonth(), datetime.GetDay(),
					datetime.GetHour(), datetime.GetMinute(), datetime.GetSecond(),
					price, size);
			}
			else if (ts.DataType == tsdBID) {
				m_dc.SetTextColor(RGB(255, 0, 0));
				sprintf(buf, "%d-%02d-%02d %02d:%02d:%02d  %.2f  %5d  BID", 
					datetime.GetYear(), datetime.GetMonth(), datetime.GetDay(),
					datetime.GetHour(), datetime.GetMinute(), datetime.GetSecond(),
					price, size);
			}
			else if (ts.DataType == tsdASK) {
				m_dc.SetTextColor(RGB(0, 255, 0));
				sprintf(buf, "%d-%02d-%02d %02d:%02d:%02d  %.2f  %5d  ASK", 
					datetime.GetYear(), datetime.GetMonth(), datetime.GetDay(),
					datetime.GetHour(), datetime.GetMinute(), datetime.GetSecond(),
					price, size);
			}

			m_dc.TextOut(x, y, buf);
			y += m_dc.GetTextExtent(buf).cy;
		}
	}

	m_dc.SelectClipRgn(NULL);
}

